#!/usr/bin/python3

def escape(str):
	result = ""
	for c in str:
		if c=='"':
			result += '\\"'
		elif c=='\\':
			result += '\\\\'
		elif ord(c)<0x20:
			result += "\\{:03o}".format(ord(c))
		else:
			result += c
	return result

def values(data, size):
	for y in range(size[1]-1, -1, -1):
		for x in range(size[0]):
			yield data[x+y*size[0]]

def values_multi(data, size):
	for y in range(size[1]-1, -1, -1):
		for x in range(size[0]):
			for c in data[x+y*size[0]]:
				yield c

def make_tex(fn, mipmap=False, srgb=False):
	from PIL import Image
	import base64
	import codecs

	img = Image.open(fn)

	fmt = "".join(img.getbands())
	if fmt=="LA":
		fmt = "LUMINANCE8_ALPHA8"
	elif fmt=="L":
		fmt = "LUMINANCE8"

	if srgb:
		fmt = "S"+fmt

	result = "storage {} {} {};\n".format(fmt, img.size[0], img.size[1])
	if mipmap:
		result += "generate_mipmap true;\n"

	if len(img.getbands())==1:
		pixels = values(img.getdata(), img.size)
	else:
		pixels = values_multi(img.getdata(), img.size)
	result += "raw_data ={}=;\n".format(codecs.decode(base64.b64encode(bytes(pixels)), "ascii"))

	return result

if __name__=="__main__":
	import os
	import argparse

	parser = argparse.ArgumentParser()
	parser.add_argument("-o", "--output", metavar="FILE", help="Output filename")
	parser.add_argument("-m", "--mipmap", action="store_true", help="Enable mipmaps")
	parser.add_argument("--srgb", action="store_const", const=True, help="Use sRGB color space")
	parser.add_argument("image")

	args = parser.parse_args()

	out_fn = args.output
	if not out_fn:
		out_fn = os.path.splitext(args.image)[0]+".tex2d"
	out = open(out_fn, "w")
	out.write(make_tex(args.image, args.mipmap, args.srgb))
